@@ -1,94 +1,43 @@
-import os
-import sys
-
-from .config import DEFAULT_CONFIG_DIR, Config
-from .compat import urlsplit, is_windows, bytes, str
+from typing import Iterable, Optional
+from urllib.parse import urlsplit
 
 
-class Environment(object):
-    """Holds information about the execution context.
-
-    Groups various aspects of the environment in a changeable object
-    and allows for mocking.
-
-    """
-
-    is_windows = is_windows
-
-    progname = os.path.basename(sys.argv[0])
-    if progname not in ['http', 'https']:
-        progname = 'http'
-
-    config_dir = DEFAULT_CONFIG_DIR
-
-    # Can be set to 0 to disable colors completely.
-    colors = 256 if '256color' in os.environ.get('TERM', '') else 88
-
-    stdin = sys.stdin
-    stdin_isatty = sys.stdin.isatty()
-
-    stdout_isatty = sys.stdout.isatty()
-    stderr_isatty = sys.stderr.isatty()
-    if is_windows:
-        # noinspection PyUnresolvedReferences
-        from colorama.initialise import wrap_stream
-        stdout = wrap_stream(sys.stdout, convert=None,
-                             strip=None, autoreset=True, wrap=True)
-        stderr = wrap_stream(sys.stderr, convert=None,
-                             strip=None, autoreset=True, wrap=True)
-    else:
-        stdout = sys.stdout
-        stderr = sys.stderr
-
-    def __init__(self, **kwargs):
-        assert all(hasattr(type(self), attr)
-                   for attr in kwargs.keys())
-        self.__dict__.update(**kwargs)
-
-    @property
-    def config(self):
-        if not hasattr(self, '_config'):
-            self._config = Config(directory=self.config_dir)
-            if self._config.is_new:
-                self._config.save()
-            else:
-                self._config.load()
-        return self._config
-
-
-class HTTPMessage(object):
+class HTTPMessage:
     """Abstract class for HTTP messages."""
 
     def __init__(self, orig):
         self._orig = orig
 
-    def iter_body(self, chunk_size):
+    def iter_body(self, chunk_size: int) -> Iterable[bytes]:
         """Return an iterator over the body."""
         raise NotImplementedError()
 
-    def iter_lines(self, chunk_size):
+    def iter_lines(self, chunk_size: int) -> Iterable[bytes]:
         """Return an iterator over the body yielding (`line`, `line_feed`)."""
         raise NotImplementedError()
 
     @property
-    def headers(self):
+    def headers(self) -> str:
         """Return a `str` with the message's headers."""
         raise NotImplementedError()
 
     @property
-    def encoding(self):
+    def encoding(self) -> Optional[str]:
         """Return a `str` with the message's encoding, if known."""
         raise NotImplementedError()
 
     @property
-    def body(self):
+    def body(self) -> bytes:
         """Return a `bytes` with the message's body."""
         raise NotImplementedError()
 
     @property
-    def content_type(self):
+    def content_type(self) -> str:
         """Return the message content type."""
-        return self._orig.headers.get('Content-Type', '')
+        ct = self._orig.headers.get('Content-Type', '')
+        if not isinstance(ct, str):
+            ct = ct.decode('utf8')
+        return ct
 
 
 class HTTPResponse(HTTPMessage):
@@ -100,15 +49,19 @@ class HTTPResponse(HTTPMessage):
     def iter_lines(self, chunk_size):
         return ((line, b'\n') for line in self._orig.iter_lines(chunk_size))
 
-    #noinspection PyProtectedMember
+    # noinspection PyProtectedMember
     @property
     def headers(self):
         original = self._orig.raw._original_response
-        status_line = 'HTTP/{version} {status} {reason}'.format(
-            version='.'.join(str(original.version)),
-            status=original.status,
-            reason=original.reason
-        )
+
+        version = {
+            9: '0.9',
+            10: '1.0',
+            11: '1.1',
+            20: '2',
+        }[original.version]
+
+        status_line = f'HTTP/{version} {original.status} {original.reason}'
         headers = [status_line]
         try:
             # `original.msg` is a `http.client.HTTPMessage` on Python 3
@@ -153,16 +106,24 @@ class HTTPRequest(HTTPMessage):
         )
 
         headers = dict(self._orig.headers)
-
-        if 'Host' not in headers:
+        if 'Host' not in self._orig.headers:
             headers['Host'] = url.netloc.split('@')[-1]
 
-        headers = ['%s: %s' % (name, value)
-                   for name, value in headers.items()]
+        headers = [
+            '%s: %s' % (
+                name,
+                value if isinstance(value, str) else value.decode('utf8')
+            )
+            for name, value in headers.items()
+        ]
 
         headers.insert(0, request_line)
+        headers = '\r\n'.join(headers).strip()
 
-        return '\r\n'.join(headers).strip()
+        if isinstance(headers, bytes):
+            # Python < 3
+            headers = headers.decode('utf8')
+        return headers
 
     @property
     def encoding(self):
