@@ -3,10 +3,26 @@ Tests for the provided defaults regarding HTTP method, and --json vs. --form.
 
 """
 from httpie.client import JSON_ACCEPT
-from utils import TestEnvironment, http, HTTP_OK
+from utils import MockEnvironment, http, HTTP_OK
 from fixtures import FILE_PATH
 
 
+def test_default_headers_case_insensitive(httpbin):
+    """
+    <https://github.com/jakubroztocil/httpie/issues/644>
+    """
+    r = http(
+        '--debug',
+        '--print=H',
+        httpbin.url + '/post',
+        'CONTENT-TYPE:application/json-patch+json',
+        'a=b',
+    )
+    assert 'CONTENT-TYPE: application/json-patch+json' in r
+    assert 'Content-Type' not in r
+
+
+# noinspection PyPep8Naming
 class TestImplicitHTTPMethod:
     def test_implicit_GET(self, httpbin):
         r = http(httpbin.url + '/get')
@@ -29,21 +45,21 @@ class TestImplicitHTTPMethod:
 
     def test_implicit_POST_stdin(self, httpbin):
         with open(FILE_PATH) as f:
-            env = TestEnvironment(stdin_isatty=False, stdin=f)
+            env = MockEnvironment(stdin_isatty=False, stdin=f)
             r = http('--form', httpbin.url + '/post', env=env)
         assert HTTP_OK in r
 
 
 class TestAutoContentTypeAndAcceptHeaders:
     """
-    Test that Accept and Content-Type correctly defaults to JSON,
-    but can still be overridden. The same with Content-Type when --form
-    -f is used.
+    Test that `Accept` and `Content-Type` correctly default to JSON,
+    but can still be overridden. The same with Content-Type when `--form`
+    `-f` is used.
 
     """
 
     def test_GET_no_data_no_auto_headers(self, httpbin):
-        # https://github.com/jkbrzt/httpie/issues/62
+        # https://github.com/jakubroztocil/httpie/issues/62
         r = http('GET', httpbin.url + '/headers')
         assert HTTP_OK in r
         assert r.json['headers']['Accept'] == '*/*'
@@ -69,12 +85,12 @@ class TestAutoContentTypeAndAcceptHeaders:
         assert r.json['headers']['Accept'] == JSON_ACCEPT
         assert r.json['headers']['Content-Type'] == 'application/json'
 
-    def test_POST_explicit_JSON_auto_JSON_accept(self, httpbin):
+    def test_POST_explicit_JSON_JSON_ACCEPT(self, httpbin):
         r = http('--json', 'POST', httpbin.url + '/post')
         assert HTTP_OK in r
         assert r.json['headers']['Accept'] == JSON_ACCEPT
         # Make sure Content-Type gets set even with no data.
-        # https://github.com/jkbrzt/httpie/issues/137
+        # https://github.com/jakubroztocil/httpie/issues/137
         assert 'application/json' in r.json['headers']['Content-Type']
 
     def test_GET_explicit_JSON_explicit_headers(self, httpbin):
@@ -97,11 +113,11 @@ class TestAutoContentTypeAndAcceptHeaders:
         assert '"Content-Type": "application/xml"' in r
 
     def test_print_only_body_when_stdout_redirected_by_default(self, httpbin):
-        env = TestEnvironment(stdin_isatty=True, stdout_isatty=False)
+        env = MockEnvironment(stdin_isatty=True, stdout_isatty=False)
         r = http('GET', httpbin.url + '/get', env=env)
         assert 'HTTP/' not in r
 
     def test_print_overridable_when_stdout_redirected(self, httpbin):
-        env = TestEnvironment(stdin_isatty=True, stdout_isatty=False)
+        env = MockEnvironment(stdin_isatty=True, stdout_isatty=False)
         r = http('--print=h', 'GET', httpbin.url + '/get', env=env)
         assert HTTP_OK in r
